@@ -24,36 +24,26 @@
 #include "seahorse-gkr-keyring.h"
 #include "seahorse-gkr-source.h"
 
+#include "seahorse-secure-buffer.h"
 #include "seahorse-widget.h"
 #include "seahorse-util.h"
 
-#include "common/seahorse-secure-buffer.h"
-
 #include <glib/gi18n.h>
 
 static void
 item_add_done (GnomeKeyringResult result, guint32 item, gpointer data)
 {
 	SeahorseWidget *swidget = SEAHORSE_WIDGET (data);
-	SeahorseOperation *op;
 
 	g_return_if_fail (swidget);
 
 	/* Clear the operation without cancelling it since it is complete */
 	seahorse_gkr_dialog_complete_request (swidget, FALSE);
-    
+
 	/* Successful. Update the listings and stuff. */
 	if (result == GNOME_KEYRING_RESULT_OK) {
-		
-		op = seahorse_source_load (SEAHORSE_SOURCE (seahorse_gkr_source_default ()));
-		
-		/* 
-		 * HACK: Major hack alert. This whole area needs some serious refactoring,
-		 * so for now we're just going to let any viewers listen in on this
-		 * operation like so:
-		 */
-		g_signal_emit_by_name (seahorse_context_for_app (), "refreshing", op);
-		g_object_unref (op);
+		seahorse_source_load_async (SEAHORSE_SOURCE (seahorse_gkr_source_default ()),
+		                            NULL, NULL, NULL);
 
 	/* Setting the default keyring failed */
 	} else if (result != GNOME_KEYRING_RESULT_CANCELLED) {     
@@ -88,7 +78,8 @@ on_add_item_response (GtkDialog *dialog, int response, SeahorseWidget *swidget)
 	const gchar *label;
 	gpointer request;
 	GArray *attributes;
-	
+	GtkTreeIter iter;
+
 	if (response == GTK_RESPONSE_HELP) {
 		seahorse_widget_show_help (swidget);
 		
@@ -102,14 +93,15 @@ on_add_item_response (GtkDialog *dialog, int response, SeahorseWidget *swidget)
 		secret = gtk_entry_get_text (GTK_ENTRY (widget));
 		
 		widget = seahorse_widget_get_widget (swidget, "item-keyring");
-#if GTK_CHECK_VERSION (2,91,2)
-		keyring = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (widget));
-#else
-		keyring = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
-#endif
+		if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter)) {
+			gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (widget)),
+			                    &iter, 0, &keyring, -1);
+		} else {
+			keyring = NULL;
+		}
 
 		attributes = gnome_keyring_attribute_list_new ();
-	    
+
 		request = gnome_keyring_item_create (keyring, GNOME_KEYRING_ITEM_NOTE, label, 
 		                                     attributes, secret, FALSE, item_add_done, 
 		                                     g_object_ref (swidget), g_object_unref);
@@ -134,8 +126,7 @@ seahorse_gkr_add_item_show (GtkWindow *parent)
 	GtkCellRenderer *cell;
 	GtkListStore *store;
 	GtkTreeIter iter;
-	gint i;
-	
+
 	swidget = seahorse_widget_new_allow_multiple ("gkr-add-item", parent);
 	g_return_if_fail (swidget);
 	
@@ -149,7 +140,7 @@ seahorse_gkr_add_item_show (GtkWindow *parent)
 	                                    
 	keyrings = seahorse_context_find_objects (NULL, SEAHORSE_GKR, SEAHORSE_USAGE_OTHER, 
 	                                          SEAHORSE_LOCATION_LOCAL);
-	for (i = 0, l = keyrings; l; l = g_list_next (l)) {
+	for (l = keyrings; l; l = g_list_next (l)) {
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter, 0, seahorse_gkr_keyring_get_name (l->data), -1);
 		if (seahorse_gkr_keyring_get_is_default (l->data))
