@@ -2,7 +2,8 @@
  * Seahorse
  * 
  * Copyright (C) 2008 Stefan Walter
- * 
+ * Copyright (C) 2011 Collabora Ltd.
+ *
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
@@ -25,245 +26,11 @@
 #include "seahorse-gkr-keyring.h"
 #include "seahorse-gkr-operation.h"
 
+#include "seahorse-progress.h"
+#include "seahorse-util.h"
+
 #include <glib/gi18n.h>
 
-/* -----------------------------------------------------------------------------
- * LIST OPERATION 
- */
- 
-
-#define SEAHORSE_TYPE_GKR_LIST_OPERATION            (seahorse_gkr_list_operation_get_type ())
-#define SEAHORSE_GKR_LIST_OPERATION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SEAHORSE_TYPE_GKR_LIST_OPERATION, SeahorseGkrListOperation))
-#define SEAHORSE_GKR_LIST_OPERATION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SEAHORSE_TYPE_GKR_LIST_OPERATION, SeahorseGkrListOperationClass))
-#define SEAHORSE_IS_GKR_LIST_OPERATION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SEAHORSE_TYPE_GKR_LIST_OPERATION))
-#define SEAHORSE_IS_GKR_LIST_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SEAHORSE_TYPE_GKR_LIST_OPERATION))
-#define SEAHORSE_GKR_LIST_OPERATION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SEAHORSE_TYPE_GKR_LIST_OPERATION, SeahorseGkrListOperationClass))
-
-typedef struct _SeahorseGkrListOperation SeahorseGkrListOperation;
-typedef struct _SeahorseGkrListOperationClass SeahorseGkrListOperationClass;
-
-struct _SeahorseGkrListOperation {
-	SeahorseOperation parent;
-	SeahorseGkrKeyring *keyring;
-	gpointer request;
-};
-
-struct _SeahorseGkrListOperationClass {
-	SeahorseOperationClass parent_class;
-};
-
-G_DEFINE_TYPE (SeahorseGkrListOperation, seahorse_gkr_list_operation, SEAHORSE_TYPE_OPERATION);
-
-static void
-keyring_operation_failed (SeahorseGkrListOperation *self, GnomeKeyringResult result)
-{
-	SeahorseOperation *op = SEAHORSE_OPERATION (self);
-	GError *err = NULL;
-    
-	g_assert (result != GNOME_KEYRING_RESULT_OK);
-	g_assert (result != GNOME_KEYRING_RESULT_CANCELLED);
-    
-	if (!seahorse_operation_is_running (op))
-		return;
-    
-	if (self->request)
-	    gnome_keyring_cancel_request (self->request);
-	self->request = NULL;
-    
-	seahorse_gkr_operation_parse_error (result, &err);
-	g_assert (err != NULL);
-    
-	seahorse_operation_mark_done (op, FALSE, err);
-}
-
-/* Remove the given key from the context */
-static void
-remove_key_from_context (gpointer kt, SeahorseObject *dummy, SeahorseSource *sksrc)
-{
-	/* This function gets called as a GHFunc on the self->checks hashtable. */
-	GQuark keyid = GPOINTER_TO_UINT (kt);
-	SeahorseObject *sobj;
-    
-	sobj = seahorse_context_get_object (SCTX_APP (), sksrc, keyid);
-	if (sobj != NULL)
-		seahorse_context_remove_object (SCTX_APP (), sobj);
-}
-
-static void
-insert_id_hashtable (SeahorseObject *object, gpointer user_data)
-{
-	g_hash_table_insert ((GHashTable*)user_data, 
-	                     GUINT_TO_POINTER (seahorse_object_get_id (object)),
-	                     GUINT_TO_POINTER (TRUE));
-}
-
-static void 
-on_list_item_ids (GnomeKeyringResult result, GList *list, SeahorseGkrListOperation *self)
-{
-	SeahorseObjectPredicate pred;
-	SeahorseGkrItem *git;
-	const gchar *keyring_name;
-	GHashTable *checks;
-	guint32 item_id;
-	GQuark id;
-	
-	if (result == GNOME_KEYRING_RESULT_CANCELLED)
-		return;
-    
-	self->request = NULL;
-
-	if (result != GNOME_KEYRING_RESULT_OK) {
-		keyring_operation_failed (self, result);
-		return;
-	}
-	
-	keyring_name = seahorse_gkr_keyring_get_name (self->keyring);
-	g_return_if_fail (keyring_name);
-
-	/* When loading new keys prepare a list of current */
-	checks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
-	seahorse_object_predicate_clear (&pred);
-	pred.source = SEAHORSE_SOURCE (self->keyring);
-	pred.type = SEAHORSE_TYPE_GKR_ITEM;
-	seahorse_context_for_objects_full (SCTX_APP (), &pred, insert_id_hashtable, checks);
-    
-	while (list) {
-		item_id = GPOINTER_TO_UINT (list->data);
-		id = seahorse_gkr_item_get_cannonical (keyring_name, item_id);
-		
-		git = SEAHORSE_GKR_ITEM (seahorse_context_get_object (SCTX_APP (), 
-		                                                      SEAHORSE_SOURCE (self->keyring), id));
-		if (!git) {
-			git = seahorse_gkr_item_new (SEAHORSE_SOURCE (self->keyring), keyring_name, item_id);
-
-			/* Add to context */ 
-			seahorse_object_set_parent (SEAHORSE_OBJECT (git), SEAHORSE_OBJECT (self->keyring));
-			seahorse_context_take_object (SCTX_APP (), SEAHORSE_OBJECT (git));
-		}
-		
-		g_hash_table_remove (checks, GUINT_TO_POINTER (id));
-		list = g_list_next (list);
-	}
-	
-	g_hash_table_foreach (checks, (GHFunc)remove_key_from_context, 
-	                      SEAHORSE_SOURCE (self->keyring));
-	
-	g_hash_table_destroy (checks);
-        
-        seahorse_operation_mark_done (SEAHORSE_OPERATION (self), FALSE, NULL);
-}
-
-static void
-on_keyring_info (GnomeKeyringResult result, GnomeKeyringInfo *info, SeahorseGkrListOperation *self)
-{
-	if (result == GNOME_KEYRING_RESULT_CANCELLED)
-		return;
-    
-	self->request = NULL;
-
-	if (result != GNOME_KEYRING_RESULT_OK) {
-		keyring_operation_failed (self, result);
-		return;
-	}
-
-	seahorse_gkr_keyring_set_info (self->keyring, info);
-	if (gnome_keyring_info_get_is_locked (info)) {
-		/* Locked, no items... */
-		on_list_item_ids (GNOME_KEYRING_RESULT_OK, NULL, self);
-	} else {
-		g_object_ref (self);
-		self->request = gnome_keyring_list_item_ids (seahorse_gkr_keyring_get_name (self->keyring), 
-		                                            (GnomeKeyringOperationGetListCallback)on_list_item_ids, 
-		                                            self, g_object_unref);
-	}
-}
-
-static SeahorseOperation*
-start_gkr_list_operation (SeahorseGkrKeyring *keyring)
-{
-	SeahorseGkrListOperation *self;
-
-	g_assert (SEAHORSE_IS_GKR_KEYRING (keyring));
-
-	self = g_object_new (SEAHORSE_TYPE_GKR_LIST_OPERATION, NULL);
-	self->keyring = keyring;
-    
-	/* Start listing of ids */
-	seahorse_operation_mark_start (SEAHORSE_OPERATION (self));
-	seahorse_operation_mark_progress (SEAHORSE_OPERATION (self), _("Listing passwords"), -1);
-	
-	g_object_ref (self);
-	self->request = gnome_keyring_get_info (seahorse_gkr_keyring_get_name (keyring), 
-	                                        (GnomeKeyringOperationGetKeyringInfoCallback)on_keyring_info, 
-	                                        self, g_object_unref);
-    
-	return SEAHORSE_OPERATION (self);
-}
-
-static void 
-seahorse_gkr_list_operation_cancel (SeahorseOperation *operation)
-{
-	SeahorseGkrListOperation *self = SEAHORSE_GKR_LIST_OPERATION (operation);    
-
-	if (self->request)
-		gnome_keyring_cancel_request (self->request);
-	self->request = NULL;
-    
-	if (seahorse_operation_is_running (operation))
-		seahorse_operation_mark_done (operation, TRUE, NULL);
-}
-
-static void 
-seahorse_gkr_list_operation_init (SeahorseGkrListOperation *self)
-{	
-	/* Everything already set to zero */
-}
-
-static void 
-seahorse_gkr_list_operation_dispose (GObject *gobject)
-{
-	SeahorseGkrListOperation *self = SEAHORSE_GKR_LIST_OPERATION (gobject);
-    
-	/* Cancel it if it's still running */
-	if (seahorse_operation_is_running (SEAHORSE_OPERATION (self)))
-		seahorse_gkr_list_operation_cancel (SEAHORSE_OPERATION (self));
-	g_assert (!seahorse_operation_is_running (SEAHORSE_OPERATION (self)));
-    
-	/* The above cancel should have stopped this */
-	g_assert (self->request == NULL);
-
-	G_OBJECT_CLASS (seahorse_gkr_list_operation_parent_class)->dispose (gobject);
-}
-
-static void 
-seahorse_gkr_list_operation_finalize (GObject *gobject)
-{
-	SeahorseGkrListOperation *self = SEAHORSE_GKR_LIST_OPERATION (gobject);
-	g_assert (!seahorse_operation_is_running (SEAHORSE_OPERATION (self)));
-    
-	/* The above cancel should have stopped this */
-	g_assert (self->request == NULL);
-
-	G_OBJECT_CLASS (seahorse_gkr_list_operation_parent_class)->finalize (gobject);
-}
-
-static void
-seahorse_gkr_list_operation_class_init (SeahorseGkrListOperationClass *klass)
-{
-	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-	SeahorseOperationClass *operation_class = SEAHORSE_OPERATION_CLASS (klass);
-	
-	seahorse_gkr_list_operation_parent_class = g_type_class_peek_parent (klass);
-	gobject_class->dispose = seahorse_gkr_list_operation_dispose;
-	gobject_class->finalize = seahorse_gkr_list_operation_finalize;
-	operation_class->cancel = seahorse_gkr_list_operation_cancel;
-    
-}
-
-/* -----------------------------------------------------------------------------------------
- * KEYRING
- */
-
 enum {
 	PROP_0,
 	PROP_SOURCE_TAG,
@@ -351,8 +118,8 @@ seahorse_gkr_keyring_realize (SeahorseObject *obj)
 	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (obj);
 	gchar *name, *markup;
 	
-	name = g_strdup_printf ("Passwords: %s", self->pv->keyring_name);
-	markup = g_markup_printf_escaped ("<b>Passwords:</b> %s", self->pv->keyring_name);
+	name = g_strdup_printf (_("Passwords: %s"), self->pv->keyring_name);
+	markup = g_markup_printf_escaped (_("<b>Passwords:</b> %s"), self->pv->keyring_name);
 	
 	g_object_set (self,
 	              "label", name,
@@ -360,8 +127,9 @@ seahorse_gkr_keyring_realize (SeahorseObject *obj)
 	              "nickname", self->pv->keyring_name,
 	              "identifier", "",
 	              "description", "",
-	              "flags", 0,
+	              "flags", SEAHORSE_FLAG_DELETABLE,
 	              "icon", "folder",
+	              "usage", SEAHORSE_USAGE_OTHER,
 	              NULL);
 	
 	g_free (name);
@@ -381,18 +149,197 @@ seahorse_gkr_keyring_refresh (SeahorseObject *obj)
 	SEAHORSE_OBJECT_CLASS (seahorse_gkr_keyring_parent_class)->refresh (obj);
 }
 
-static SeahorseOperation*
-seahorse_gkr_keyring_delete (SeahorseObject *obj)
+typedef struct {
+	SeahorseGkrKeyring *keyring;
+	gpointer request;
+	GCancellable *cancellable;
+	gulong cancelled_sig;
+} keyring_load_closure;
+
+static void
+keyring_load_free (gpointer data)
 {
-	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (obj);
-	return seahorse_gkr_operation_delete_keyring (self);
+	keyring_load_closure *closure = data;
+	if (closure->cancellable && closure->cancelled_sig)
+		g_signal_handler_disconnect (closure->cancellable,
+		                             closure->cancelled_sig);
+	if (closure->cancellable)
+		g_object_unref (closure->cancellable);
+	g_clear_object (&closure->keyring);
+	g_assert (!closure->request);
+	g_free (closure);
 }
 
-static SeahorseOperation*
-seahorse_gkr_keyring_load (SeahorseSource *src)
+/* Remove the given key from the context */
+static void
+remove_key_from_context (gpointer kt, SeahorseObject *dummy, SeahorseSource *sksrc)
 {
-	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (src);
-	return start_gkr_list_operation (self);
+	/* This function gets called as a GHFunc on the self->checks hashtable. */
+	GQuark keyid = GPOINTER_TO_UINT (kt);
+	SeahorseObject *sobj;
+
+	sobj = seahorse_context_get_object (SCTX_APP (), sksrc, keyid);
+	if (sobj != NULL)
+		seahorse_context_remove_object (SCTX_APP (), sobj);
+}
+
+static void
+insert_id_hashtable (SeahorseObject *object, gpointer user_data)
+{
+	g_hash_table_insert ((GHashTable*)user_data,
+	                     GUINT_TO_POINTER (seahorse_object_get_id (object)),
+	                     GUINT_TO_POINTER (TRUE));
+}
+
+static void
+on_keyring_load_list_item_ids (GnomeKeyringResult result,
+                               GList *list,
+                               gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	keyring_load_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	SeahorseObjectPredicate pred;
+	SeahorseGkrItem *git;
+	const gchar *keyring_name;
+	GError *error = NULL;
+	GHashTable *checks;
+	guint32 item_id;
+	GQuark id;
+
+	closure->request = NULL;
+
+	if (seahorse_gkr_propagate_error (result, &error)) {
+		g_simple_async_result_take_error (res, error);
+		g_simple_async_result_complete_in_idle (res);
+		return;
+	}
+
+	keyring_name = seahorse_gkr_keyring_get_name (closure->keyring);
+	g_return_if_fail (keyring_name);
+
+	/* When loading new keys prepare a list of current */
+	checks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
+	seahorse_object_predicate_clear (&pred);
+	pred.source = SEAHORSE_SOURCE (closure->keyring);
+	pred.type = SEAHORSE_TYPE_GKR_ITEM;
+	seahorse_context_for_objects_full (SCTX_APP (), &pred, insert_id_hashtable, checks);
+
+	while (list) {
+		item_id = GPOINTER_TO_UINT (list->data);
+		id = seahorse_gkr_item_get_cannonical (keyring_name, item_id);
+
+		git = SEAHORSE_GKR_ITEM (seahorse_context_get_object (seahorse_context_instance (),
+		                                                      SEAHORSE_SOURCE (closure->keyring), id));
+		if (!git) {
+			git = seahorse_gkr_item_new (SEAHORSE_SOURCE (closure->keyring), keyring_name, item_id);
+
+			/* Add to context */
+			seahorse_object_set_parent (SEAHORSE_OBJECT (git), SEAHORSE_OBJECT (closure->keyring));
+			seahorse_context_take_object (seahorse_context_instance (), SEAHORSE_OBJECT (git));
+		}
+
+		g_hash_table_remove (checks, GUINT_TO_POINTER (id));
+		list = g_list_next (list);
+	}
+
+	g_hash_table_foreach (checks, (GHFunc)remove_key_from_context,
+	                      SEAHORSE_SOURCE (closure->keyring));
+
+	g_hash_table_destroy (checks);
+
+	seahorse_progress_end (closure->cancellable, res);
+	g_simple_async_result_complete_in_idle (res);
+}
+
+static void
+on_keyring_load_get_info (GnomeKeyringResult result,
+                          GnomeKeyringInfo *info,
+                          gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	keyring_load_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	GError *error = NULL;
+	const gchar *name;
+
+	closure->request = NULL;
+
+	if (seahorse_gkr_propagate_error (result, &error)) {
+		g_simple_async_result_take_error (res, error);
+		g_simple_async_result_complete_in_idle (res);
+
+	} else {
+		seahorse_gkr_keyring_set_info (closure->keyring, info);
+
+		/* Locked, no items... */
+		if (gnome_keyring_info_get_is_locked (info)) {
+			on_keyring_load_list_item_ids (GNOME_KEYRING_RESULT_OK, NULL, res);
+
+		} else {
+			name = seahorse_gkr_keyring_get_name (closure->keyring);
+			closure->request = gnome_keyring_list_item_ids (name,
+			                                                on_keyring_load_list_item_ids,
+			                                                g_object_ref (res), g_object_unref);
+		}
+	}
+}
+
+static void
+on_keyring_load_cancelled (GCancellable *cancellable,
+                           gpointer user_data)
+{
+	keyring_load_closure *closure = user_data;
+
+	if (closure->request)
+		gnome_keyring_cancel_request (closure->request);
+}
+
+static void
+seahorse_gkr_keyring_load_async (SeahorseSource *source,
+                                 GCancellable *cancellable,
+                                 GAsyncReadyCallback callback,
+                                 gpointer user_data)
+{
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (source);
+	keyring_load_closure *closure;
+	GSimpleAsyncResult *res;
+#if TODO
+/* cancellable = g_cancellable_new ();
+g_cancellable_cancel (cancellable); */
+#endif
+
+	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
+	                                 seahorse_gkr_keyring_load_async);
+
+	closure = g_new0 (keyring_load_closure, 1);
+	closure->keyring = g_object_ref (self);
+	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+	g_simple_async_result_set_op_res_gpointer (res, closure, keyring_load_free);
+
+	closure->request = gnome_keyring_get_info (seahorse_gkr_keyring_get_name (self),
+	                                           on_keyring_load_get_info,
+	                                           g_object_ref (res), g_object_unref);
+
+	if (cancellable)
+		closure->cancelled_sig = g_cancellable_connect (cancellable,
+		                                                G_CALLBACK (on_keyring_load_cancelled),
+		                                                closure, NULL);
+	seahorse_progress_prep_and_begin (cancellable, res, NULL);
+
+	g_object_unref (res);
+}
+
+static gboolean
+seahorse_gkr_keyring_load_finish (SeahorseSource *source,
+                                  GAsyncResult *result,
+                                  GError **error)
+{
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (source),
+	                      seahorse_gkr_keyring_load_async), FALSE);
+
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+		return FALSE;
+
+	return TRUE;
 }
 
 static GObject* 
@@ -509,12 +456,11 @@ seahorse_gkr_keyring_class_init (SeahorseGkrKeyringClass *klass)
 	gobject_class->finalize = seahorse_gkr_keyring_finalize;
 	gobject_class->set_property = seahorse_gkr_keyring_set_property;
 	gobject_class->get_property = seahorse_gkr_keyring_get_property;
-    
+
 	seahorse_class = SEAHORSE_OBJECT_CLASS (klass);
 	seahorse_class->realize = seahorse_gkr_keyring_realize;
 	seahorse_class->refresh = seahorse_gkr_keyring_refresh;
-	seahorse_class->delete = seahorse_gkr_keyring_delete;
-	
+
 	g_object_class_override_property (gobject_class, PROP_SOURCE_TAG, "source-tag");
 	g_object_class_override_property (gobject_class, PROP_SOURCE_LOCATION, "source-location");
 
@@ -534,7 +480,8 @@ seahorse_gkr_keyring_class_init (SeahorseGkrKeyringClass *klass)
 static void 
 seahorse_source_iface (SeahorseSourceIface *iface)
 {
-	iface->load = seahorse_gkr_keyring_load;
+	iface->load_async = seahorse_gkr_keyring_load_async;
+	iface->load_finish = seahorse_gkr_keyring_load_finish;
 }
 
 /* -----------------------------------------------------------------------------
