@@ -30,9 +30,7 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#if GTK_CHECK_VERSION (2,90,0)
 #include <gdk/gdkkeysyms-compat.h>
-#endif
 
 #include "egg-datetime.h"
 
@@ -235,11 +233,7 @@ static void  egg_datetime_get_property (GObject    *object,
                    GValue        *value,
                    GParamSpec    *pspec);
 
-#if GTK_CHECK_VERSION (2, 90, 0)
 static void  egg_datetime_destroy      (GtkWidget *widget);
-#else
-static void  egg_datetime_destroy      (GtkObject *widget);
-#endif
 
 static void  egg_datetime_finalize     (GObject    *object);
 
@@ -314,11 +308,7 @@ egg_datetime_class_init (EggDateTimeClass *klass)
         o_class->set_property = egg_datetime_set_property;
         o_class->get_property = egg_datetime_get_property;
 
-#if GTK_CHECK_VERSION (2,90,0)
-        ((GtkWidgetClass*)klass)->destroy     = egg_datetime_destroy;
-#else
-        ((GtkObjectClass*)klass)->destroy     = egg_datetime_destroy;
-#endif
+    ((GtkWidgetClass*)klass)->destroy     = egg_datetime_destroy;
 
 
    /* Properties */
@@ -746,11 +736,7 @@ egg_datetime_get_property (GObject  *object,
 }
 
 static void
-#if GTK_CHECK_VERSION (2, 90, 0)
 egg_datetime_destroy (GtkWidget *widget)
-#else
-egg_datetime_destroy (GtkObject *widget)
-#endif
 {
    EggDateTime *edt = EGG_DATETIME (widget);
    EggDateTimePrivate *priv = edt->priv;
@@ -765,13 +751,8 @@ egg_datetime_destroy (GtkObject *widget)
       priv->time_popup = NULL;
    }
 
-#if GTK_CHECK_VERSION (2, 90, 0)
 	if (GTK_WIDGET_CLASS (parent_class)->destroy)
 		(* GTK_WIDGET_CLASS (parent_class)->destroy) (widget);
-#else
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (widget);
-#endif
 }
 
 static void
@@ -846,7 +827,7 @@ popup_position (GtkWidget *widget, GtkWindow *popup)
    GtkRequisition requisition;
    gint x, y, width, height;
 
-   gtk_widget_size_request (GTK_WIDGET (popup), &requisition);
+   gtk_widget_get_preferred_size (GTK_WIDGET (popup), &requisition, NULL);
    gdk_window_get_origin (gtk_widget_get_window (widget), &x, &y);
 
    gtk_widget_get_allocation (widget, &allocation);
@@ -876,12 +857,14 @@ popup_show (GtkWindow *popup)
    gtk_grab_add (GTK_WIDGET (popup));
 
    cursor = gdk_cursor_new (GDK_ARROW);
-   gdk_pointer_grab (gtk_widget_get_window (GTK_WIDGET (popup)), TRUE,
-           (GDK_BUTTON_PRESS_MASK
-            | GDK_BUTTON_RELEASE_MASK
-            | GDK_POINTER_MOTION_MASK),
-           NULL, cursor, GDK_CURRENT_TIME);
-   gdk_cursor_unref (cursor);
+   gdk_device_grab (gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (gdk_display_get_default ())),
+                    gtk_widget_get_window (GTK_WIDGET (popup)),
+                    GDK_OWNERSHIP_APPLICATION,
+                    TRUE,
+                    (GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK),
+                    cursor,
+                    GDK_CURRENT_TIME);
+   g_object_unref (cursor);
 }
 
 static void
@@ -889,7 +872,8 @@ popup_hide (GtkWindow *popup)
 {
    gtk_widget_hide (GTK_WIDGET (popup));
    gtk_grab_remove (GTK_WIDGET (popup));
-   gdk_pointer_ungrab (GDK_CURRENT_TIME);
+   gdk_device_ungrab (gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (gdk_display_get_default ())),
+                      GDK_CURRENT_TIME);
 }
 
 /*
