@@ -2,7 +2,8 @@
  * Seahorse
  * 
  * Copyright (C) 2008 Stefan Walter
- * 
+ * Copyright (C) 2011 Collabora Ltd.
+ *
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
@@ -30,7 +31,26 @@
 
 #include <glib/gi18n.h>
 
-enum {
+/**
+ * _SeahorseObjectProps:
+ * @PROP_0:
+ * @PROP_CONTEXT:
+ * @PROP_SOURCE:
+ * @PROP_PREFERRED:
+ * @PROP_PARENT:
+ * @PROP_ID:
+ * @PROP_TAG:
+ * @PROP_LABEL:
+ * @PROP_MARKUP:
+ * @PROP_NICKNAME:
+ * @PROP_DESCRIPTION:
+ * @PROP_ICON:
+ * @PROP_IDENTIFIER:
+ * @PROP_LOCATION:
+ * @PROP_USAGE:
+ * @PROP_FLAGS:
+ */
+enum _SeahorseObjectProps {
 	PROP_0,
 	PROP_CONTEXT,
 	PROP_SOURCE,
@@ -49,6 +69,32 @@ enum {
 	PROP_FLAGS
 };
 
+/**
+ * _SeahorseObjectPrivate:
+ * @id: the id of the object
+ * @tag: DBUS: "ktype"
+ * @tag_explicit: If TRUE the tag will not be set automatically
+ * @source: Where did the object come from ?
+ * @context:
+ * @preferred: Points to the object to prefer over this one
+ * @parent: the object's parent
+ * @children: a list of children the object has
+ * @label: DBUS: "display-name"
+ * @markup: Markup text
+ * @markup_explicit: If TRUE the markup will not be set automatically
+ * @nickname: DBUS: "simple-name"
+ * @nickname_explicit: If TRUE the nickname will not be set automatically
+ * @description:  Description text DBUS: "key-desc"
+ * @description_explicit: If TRUE the description will not be set automatically
+ * @icon: DBUS: "stock-id"
+ * @identifier: DBUS: "key-id", "display-id", "raw-id"
+ * @identifier_explicit:
+ * @location: describes the loaction of the object (local, remte, invalid...)
+ * @usage: DBUS: "etype"
+ * @flags:
+ * @realizing: set while the object is realizing
+ * @realized: set as soon as the object is realized
+ */
 struct _SeahorseObjectPrivate {
 	GQuark id;
 	GQuark tag;
@@ -60,21 +106,21 @@ struct _SeahorseObjectPrivate {
 	SeahorseObject *parent;
 	GList *children;
 	
-	gchar *label;
-	gchar *markup;
-	gboolean markup_explicit;
-	gchar *nickname;
-	gboolean nickname_explicit;
-	gchar *description;
-	gboolean description_explicit;
-	gchar *icon;
-	gchar *identifier;
-	gboolean identifier_explicit;
-	
-	SeahorseLocation location;
-	SeahorseUsage usage;
-	guint flags;
-	
+    gchar *label;             
+    gchar *markup;
+    gboolean markup_explicit;
+    gchar *nickname;
+    gboolean nickname_explicit;
+    gchar *description;
+    gboolean description_explicit;
+    gchar *icon;
+    gchar *identifier;
+    gboolean identifier_explicit;
+
+    SeahorseLocation location;
+    SeahorseUsage usage;
+    guint flags;
+
 	gboolean realized;
 	gboolean realizing;
 };
@@ -85,6 +131,14 @@ G_DEFINE_TYPE (SeahorseObject, seahorse_object, G_TYPE_OBJECT);
  * INTERNAL 
  */
 
+/**
+ * register_child:
+ * @self: The parent
+ * @child: The new child
+ *
+ *
+ * Sets @child as a child of @self
+ */
 static void 
 register_child (SeahorseObject* self, SeahorseObject* child) 
 {
@@ -97,6 +151,14 @@ register_child (SeahorseObject* self, SeahorseObject* child)
 	self->pv->children = g_list_append (self->pv->children, child);
 }
 
+/**
+ * unregister_child:
+ * @self: The parent
+ * @child: child to remove
+ *
+ *
+ * removes @child from the children list in @self
+ */
 static void 
 unregister_child (SeahorseObject* self, SeahorseObject* child) 
 {
@@ -109,6 +171,15 @@ unregister_child (SeahorseObject* self, SeahorseObject* child)
 	self->pv->children = g_list_remove (self->pv->children, child);
 }
 
+/**
+ * set_string_storage:
+ * @value: The value to store
+ * @storage: The datastore to write the value to
+ *
+ * When storing, new memory will be allocated and the value copied
+ *
+ * Returns: TRUE if the value has been set new, FALSE else
+ */
 static gboolean
 set_string_storage (const gchar *value, gchar **storage)
 {
@@ -126,6 +197,15 @@ set_string_storage (const gchar *value, gchar **storage)
 	return FALSE;
 }
 
+/**
+ * take_string_storage:
+ * @value: The value to store
+ * @storage: The datastore to write the value to
+ *
+ * When storing, the pointer to value is used. No new memory will be allocated
+ *
+ * Returns: TRUE if the value has been set new, FALSE else
+ */
 static gboolean
 take_string_storage (gchar *value, gchar **storage)
 {
@@ -144,6 +224,13 @@ take_string_storage (gchar *value, gchar **storage)
 	return FALSE;
 }
 
+/**
+ * recalculate_id:
+ * @self: object to calculate for
+ *
+ * recalculates tag and identifier from id
+ *
+ */
 static void
 recalculate_id (SeahorseObject *self)
 {
@@ -189,6 +276,13 @@ recalculate_id (SeahorseObject *self)
 	}
 }
 
+/**
+ * recalculate_label:
+ * @self: object to recalculate label for
+ *
+ * Recalculates nickname and markup from the label
+ *
+ */
 static void
 recalculate_label (SeahorseObject *self)
 {
@@ -204,6 +298,14 @@ recalculate_label (SeahorseObject *self)
 	}
 }
 
+/**
+ * recalculate_usage:
+ * @self: The #SeahorseObject to recalculate the usage decription for
+ *
+ * Basing on the usage identifiere this function will calculate a usage
+ * description and store it in the object.
+ *
+ */
 static void
 recalculate_usage (SeahorseObject *self)
 {
@@ -225,6 +327,10 @@ recalculate_usage (SeahorseObject *self)
 			desc = _("Credentials");
 			break;
 		case SEAHORSE_USAGE_IDENTITY:
+		    /*
+		     * Translators: "This object is a means of storing items such as 
+		     * name, email address, etc. that make up one's digital identity.
+		     */
 			desc = _("Identity");
 			break;
 		default:
@@ -242,6 +348,13 @@ recalculate_usage (SeahorseObject *self)
  * OBJECT 
  */
 
+/**
+ * seahorse_object_init:
+ * @self: The object to initialise
+ *
+ * Initialises the object with default data
+ *
+ */
 static void
 seahorse_object_init (SeahorseObject *self)
 {
@@ -256,12 +369,20 @@ seahorse_object_init (SeahorseObject *self)
 	self->pv->usage = SEAHORSE_USAGE_NONE;
 }
 
+
+/**
+ * seahorse_object_dispose:
+ * @obj: A #SeahorseObject to dispose
+ *
+ * Before this object is disposed, all it's children get new parents
+ *
+ */
 static void
 seahorse_object_dispose (GObject *obj)
 {
 	SeahorseObject *self = SEAHORSE_OBJECT (obj);
 	SeahorseObject *parent;
-	GList *l;
+	GList *l, *children;
 	
 	if (self->pv->context != NULL) {
 		seahorse_context_remove_object (self->pv->context, self);
@@ -287,11 +408,13 @@ seahorse_object_dispose (GObject *obj)
 	parent = self->pv->parent;
 	if (parent)
 		g_object_ref (parent);
-	
-	for (l = self->pv->children; l; l = g_list_next (l)) {
+
+	children = g_list_copy (self->pv->children);
+	for (l = children; l; l = g_list_next (l)) {
 		g_return_if_fail (SEAHORSE_IS_OBJECT (l->data));
 		seahorse_object_set_parent (l->data, parent);
 	}
+	g_list_free (children);
 
 	if (parent)
 		g_object_unref (parent);
@@ -304,6 +427,11 @@ seahorse_object_dispose (GObject *obj)
 	G_OBJECT_CLASS (seahorse_object_parent_class)->dispose (obj);	
 }
 
+/**
+ * seahorse_object_finalize:
+ * @obj: #SeahorseObject to finalize
+ *
+ */
 static void
 seahorse_object_finalize (GObject *obj)
 {
@@ -336,6 +464,17 @@ seahorse_object_finalize (GObject *obj)
 	G_OBJECT_CLASS (seahorse_object_parent_class)->finalize (obj);
 }
 
+
+/**
+ * seahorse_object_get_property:
+ * @obj: The object to get the property for
+ * @prop_id: The property requested
+ * @value: out - the value as #GValue
+ * @pspec: a #GParamSpec for the warning
+ *
+ * Returns: The property of the object @obj defined by the id @prop_id in @value.
+ *
+ */
 static void
 seahorse_object_get_property (GObject *obj, guint prop_id, GValue *value, 
                            GParamSpec *pspec)
@@ -394,6 +533,16 @@ seahorse_object_get_property (GObject *obj, guint prop_id, GValue *value,
 	}
 }
 
+/**
+ * seahorse_object_set_property:
+ * @obj: Object to set the property in
+ * @prop_id: the property to set
+ * @value: the value to set
+ * @pspec: To be used for warnings. #GParamSpec
+ *
+ * Sets a property in this object
+ *
+ */
 static void
 seahorse_object_set_property (GObject *obj, guint prop_id, const GValue *value, 
                               GParamSpec *pspec)
@@ -495,10 +644,6 @@ seahorse_object_set_property (GObject *obj, guint prop_id, const GValue *value,
 		break;
 	case PROP_FLAGS:
 		flags = g_value_get_uint (value);
-		if (SEAHORSE_OBJECT_GET_CLASS (obj)->delete)
-			flags |= SEAHORSE_FLAG_DELETABLE;
-		else 
-			flags &= ~SEAHORSE_FLAG_DELETABLE;
 		if (flags != self->pv->flags) {
 			self->pv->flags = flags;
 			g_object_notify (obj, "flags");
@@ -510,6 +655,13 @@ seahorse_object_set_property (GObject *obj, guint prop_id, const GValue *value,
 	}
 }
 
+/**
+ * seahorse_object_real_realize:
+ * @self: The object
+ *
+ * To be overridden
+ *
+ */
 static void 
 seahorse_object_real_realize (SeahorseObject *self)
 {
@@ -521,6 +673,13 @@ seahorse_object_real_realize (SeahorseObject *self)
 	self->pv->realized = TRUE;
 }
 
+/**
+ * seahorse_object_real_refresh:
+ * @self: The object
+ *
+ * To be overridden
+ *
+ */
 static void 
 seahorse_object_real_refresh (SeahorseObject *self)
 {
@@ -530,6 +689,13 @@ seahorse_object_real_refresh (SeahorseObject *self)
 	 */
 }
 
+/**
+ * seahorse_object_class_init:
+ * @klass: the object class
+ *
+ * Initialises the object class
+ *
+ */
 static void
 seahorse_object_class_init (SeahorseObjectClass *klass)
 {
@@ -611,15 +777,25 @@ seahorse_object_class_init (SeahorseObjectClass *klass)
  * PUBLIC 
  */
 
+/**
+ * seahorse_object_get_id:
+ * @self: Object
+ *
+ * Returns: the id of the object @self
+ */
 GQuark
 seahorse_object_get_id (SeahorseObject *self)
 {
 	g_return_val_if_fail (SEAHORSE_IS_OBJECT (self), 0);
-	if (!self->pv->id)
-		seahorse_object_realize (self);
 	return self->pv->id;
 }
 
+/**
+ * seahorse_object_get_tag:
+ * @self: Object
+ *
+ * Returns: the tag of the object @self
+ */
 GQuark
 seahorse_object_get_tag (SeahorseObject *self)
 {
@@ -629,6 +805,12 @@ seahorse_object_get_tag (SeahorseObject *self)
 	return self->pv->tag;	
 }
 
+/**
+ * seahorse_object_get_source:
+ * @self: Object
+ *
+ * Returns: the source of the object @self
+ */
 SeahorseSource*
 seahorse_object_get_source (SeahorseObject *self)
 {
@@ -636,6 +818,14 @@ seahorse_object_get_source (SeahorseObject *self)
 	return self->pv->source;	
 }
 
+
+/**
+ * seahorse_object_set_source:
+ * @self: The object to set a new source for
+ * @value: The source to set
+ *
+ * sets the source for the object
+ */
 void
 seahorse_object_set_source (SeahorseObject *self, SeahorseSource *value)
 {
@@ -654,6 +844,12 @@ seahorse_object_set_source (SeahorseObject *self, SeahorseSource *value)
 	g_object_notify (G_OBJECT (self), "source");
 }
 
+/**
+ * seahorse_object_get_context:
+ * @self: Object
+ *
+ * Returns: the context of the object @self
+ */
 SeahorseContext*
 seahorse_object_get_context (SeahorseObject *self)
 {
@@ -661,6 +857,12 @@ seahorse_object_get_context (SeahorseObject *self)
 	return self->pv->context;
 }
 
+/**
+ * seahorse_object_get_preferred:
+ * @self: Object
+ *
+ * Returns: the preferred of the object @self
+ */
 SeahorseObject*
 seahorse_object_get_preferred (SeahorseObject *self)
 {
@@ -668,6 +870,13 @@ seahorse_object_get_preferred (SeahorseObject *self)
 	return self->pv->preferred;
 }
 
+/**
+ * seahorse_object_set_preferred:
+ * @self: the object to set the preferred object for
+ * @value: the preferred object
+ *
+ *
+ */
 void
 seahorse_object_set_preferred (SeahorseObject *self, SeahorseObject *value)
 {
@@ -686,6 +895,12 @@ seahorse_object_set_preferred (SeahorseObject *self, SeahorseObject *value)
 	g_object_notify (G_OBJECT (self), "preferred");
 }
 
+/**
+ * seahorse_object_get_parent:
+ * @self: Object
+ *
+ * Returns: the parent of the object @self
+ */
 SeahorseObject*
 seahorse_object_get_parent (SeahorseObject *self)
 {
@@ -693,6 +908,13 @@ seahorse_object_get_parent (SeahorseObject *self)
 	return self->pv->parent;
 }
 
+/**
+ * seahorse_object_set_parent:
+ * @self: the child
+ * @value: the parent
+ *
+ * register @value as the parent of @self:
+ */
 void
 seahorse_object_set_parent (SeahorseObject *self, SeahorseObject *value)
 {
@@ -714,7 +936,13 @@ seahorse_object_set_parent (SeahorseObject *self, SeahorseObject *value)
 
 	g_object_notify (G_OBJECT (self), "parent");
 }
- 
+
+/**
+ * seahorse_object_get_children:
+ * @self: Object
+ *
+ * Returns: the children of the object @self
+ */
 GList*
 seahorse_object_get_children (SeahorseObject *self)
 {
@@ -723,6 +951,13 @@ seahorse_object_get_children (SeahorseObject *self)
 	return g_list_copy (self->pv->children);
 }
 
+/**
+ * seahorse_object_get_nth_child:
+ * @self: Object
+ * @index: the number of the child to return
+ *
+ * Returns: the child number @index
+ */
 SeahorseObject*
 seahorse_object_get_nth_child (SeahorseObject *self, guint index)
 {
@@ -730,6 +965,12 @@ seahorse_object_get_nth_child (SeahorseObject *self, guint index)
 	return SEAHORSE_OBJECT (g_list_nth_data (self->pv->children, index));
 }
 
+/**
+ * seahorse_object_get_label:
+ * @self: Object
+ *
+ * Returns: the label of the object @self
+ */
 const gchar*
 seahorse_object_get_label (SeahorseObject *self)
 {
@@ -738,6 +979,12 @@ seahorse_object_get_label (SeahorseObject *self)
 	return self->pv->label;
 }
 
+/**
+ * seahorse_object_get_markup:
+ * @self: Object
+ *
+ * Returns: the markup of the object @self
+ */
 const gchar*
 seahorse_object_get_markup (SeahorseObject *self)
 {
@@ -746,6 +993,12 @@ seahorse_object_get_markup (SeahorseObject *self)
 	return self->pv->markup;
 }
 
+/**
+ * seahorse_object_get_nickname:
+ * @self: Object
+ *
+ * Returns: the nickname of the object @self
+ */
 const gchar*
 seahorse_object_get_nickname (SeahorseObject *self)
 {
@@ -754,6 +1007,12 @@ seahorse_object_get_nickname (SeahorseObject *self)
 	return self->pv->nickname;
 }
 
+/**
+ * seahorse_object_get_description:
+ * @self: Object
+ *
+ * Returns: the description of the object @self
+ */
 const gchar*
 seahorse_object_get_description (SeahorseObject *self)
 {
@@ -762,6 +1021,12 @@ seahorse_object_get_description (SeahorseObject *self)
 	return self->pv->description;
 }
 
+/**
+ * seahorse_object_get_icon:
+ * @self: Object
+ *
+ * Returns: the icon of the object @self
+ */
 const gchar*
 seahorse_object_get_icon (SeahorseObject *self)
 {
@@ -770,6 +1035,12 @@ seahorse_object_get_icon (SeahorseObject *self)
 	return self->pv->icon;
 }
 
+/**
+ * seahorse_object_get_identifier:
+ * @self: Object
+ *
+ * Returns: the identifier of the object @self
+ */
 const gchar*
 seahorse_object_get_identifier (SeahorseObject *self)
 {
@@ -778,6 +1049,12 @@ seahorse_object_get_identifier (SeahorseObject *self)
 	return self->pv->identifier;	
 }
 
+/**
+ * seahorse_object_get_location:
+ * @self: Object
+ *
+ * Returns: the location of the object @self
+ */
 SeahorseLocation
 seahorse_object_get_location (SeahorseObject *self)
 {
@@ -787,6 +1064,12 @@ seahorse_object_get_location (SeahorseObject *self)
 	return self->pv->location;
 }
 
+/**
+ * seahorse_object_get_usage:
+ * @self: Object
+ *
+ * Returns: the usage of the object @self
+ */
 SeahorseUsage
 seahorse_object_get_usage (SeahorseObject *self)
 {
@@ -796,6 +1079,12 @@ seahorse_object_get_usage (SeahorseObject *self)
 	return self->pv->usage;	
 }
 
+/**
+ * seahorse_object_get_flags:
+ * @self: Object
+ *
+ * Returns: the flags of the object @self
+ */
 guint
 seahorse_object_get_flags (SeahorseObject *self)
 {
@@ -804,6 +1093,16 @@ seahorse_object_get_flags (SeahorseObject *self)
 	return self->pv->flags;	
 }
 
+/**
+ * seahorse_object_lookup_property:
+ * @self: the object to look up the property
+ * @field: the field to lookup
+ * @value: the returned value
+ *
+ * Looks up the property @field in the object @self and returns it in @value
+ *
+ * Returns: TRUE if a property was found
+ */
 gboolean
 seahorse_object_lookup_property (SeahorseObject *self, const gchar *field, GValue *value)
 {
@@ -832,6 +1131,8 @@ seahorse_object_lookup_property (SeahorseObject *self, const gchar *field, GValu
 			field = "identifier";
 		else if (g_str_equal (field, "stock-id"))
 			field = "icon";
+		else if (g_str_equal (field, "raw-id"))
+			field = "identifier";
 		else 
 			return FALSE;
 	
@@ -846,6 +1147,13 @@ seahorse_object_lookup_property (SeahorseObject *self, const gchar *field, GValu
 	return TRUE; 
 }
 
+/**
+ * seahorse_object_realize:
+ * @self: the object to realize
+ *
+ *
+ * Realizes an object. Calls the klass method
+ */
 void
 seahorse_object_realize (SeahorseObject *self)
 {
@@ -862,6 +1170,13 @@ seahorse_object_realize (SeahorseObject *self)
 	self->pv->realizing = FALSE;
 }
 
+/**
+ * seahorse_object_refresh:
+ * @self: object to refresh
+ *
+ * calls the class refresh function
+ *
+ */
 void
 seahorse_object_refresh (SeahorseObject *self)
 {
@@ -872,16 +1187,15 @@ seahorse_object_refresh (SeahorseObject *self)
 	(klass->refresh) (self);
 }
 
-SeahorseOperation*
-seahorse_object_delete (SeahorseObject *self)
-{
-	SeahorseObjectClass *klass;
-	g_return_val_if_fail (SEAHORSE_IS_OBJECT (self), NULL);
-	klass = SEAHORSE_OBJECT_GET_CLASS (self);
-	g_return_val_if_fail (klass->delete, NULL);
-	return (klass->delete) (self);
-}
-
+/**
+ * seahorse_object_predicate_match:
+ * @self: the object to test
+ * @obj: The predicate to match
+ *
+ * matches a seahorse object and a predicate
+ *
+ * Returns: FALSE if predicate does not match the #SeahorseObject, TRUE else
+ */
 gboolean 
 seahorse_object_predicate_match (SeahorseObjectPredicate *self, SeahorseObject* obj) 
 {
@@ -917,6 +1231,12 @@ seahorse_object_predicate_match (SeahorseObjectPredicate *self, SeahorseObject*
 	return TRUE;
 }
 
+/**
+ * seahorse_object_predicate_clear:
+ * @self: The predicate to clean
+ *
+ * Clears a seahorse predicate (#SeahorseObjectPredicate)
+ */
 void
 seahorse_object_predicate_clear (SeahorseObjectPredicate *self)
 {
