@@ -2,7 +2,8 @@
  * Seahorse
  * 
  * Copyright (C) 2008 Stefan Walter
- * 
+ * Copyright (C) 2011 Collabora Ltd.
+ *
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
@@ -73,8 +74,6 @@ struct _SeahorseObjectClass {
 	void (*realize) (SeahorseObject *self);
 	
 	void (*refresh) (SeahorseObject *self);
-	
-	SeahorseOperation* (*delete) (SeahorseObject *self);
 };
 
 GType               seahorse_object_get_type               (void);
@@ -85,8 +84,6 @@ void                seahorse_object_realize                (SeahorseObject *self
 
 void                seahorse_object_refresh                (SeahorseObject *self);
 
-SeahorseOperation*  seahorse_object_delete                 (SeahorseObject *self);
-
 GQuark              seahorse_object_get_id                 (SeahorseObject *self);
 
 GQuark              seahorse_object_get_tag                (SeahorseObject *self);
