@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2005 Stefan Walter
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,12 +26,11 @@
 #include <glib/gi18n.h>
 
 #include "seahorse-context.h"
+#include "seahorse-object-list.h"
 #include "seahorse-source.h"
 #include "seahorse-gtkstock.h"
 #include "seahorse-util.h"
 
-#include "common/seahorse-object-list.h"
-
 #include "pgp/seahorse-pgp-key.h"
 #include "pgp/seahorse-gpgme.h"
 #include "pgp/seahorse-gpgme-key-op.h"
@@ -68,7 +68,7 @@ struct _SeahorseGpgmeKeyPrivate {
 static gboolean 
 load_gpgme_key (GQuark id, int mode, int secret, gpgme_key_t *key)
 {
-	GError *err = NULL;
+	GError *error = NULL;
 	gpgme_ctx_t ctx;
 	gpgme_error_t gerr;
 	
@@ -81,10 +81,10 @@ load_gpgme_key (GQuark id, int mode, int secret, gpgme_key_t *key)
 	}
 
 	gpgme_release (ctx);
-	
-	if (!GPG_IS_OK (gerr)) {
-		seahorse_gpgme_to_error (gerr, &err);
-		g_message ("couldn't load GPGME key: %s", err->message);
+
+	if (seahorse_gpgme_propagate_error (gerr, &error)) {
+		g_message ("couldn't load GPGME key: %s", error->message);
+		g_clear_error (&error);
 		return FALSE;
 	}
 	
@@ -324,7 +324,7 @@ seahorse_gpgme_key_realize (SeahorseObject *obj)
 	realize_subkeys (self);
 
 	/* The flags */
-	flags = SEAHORSE_FLAG_EXPORTABLE;
+	flags = SEAHORSE_FLAG_EXPORTABLE | SEAHORSE_FLAG_DELETABLE;
 
 	if (!self->pv->pubkey->disabled && !self->pv->pubkey->expired && 
 	    !self->pv->pubkey->revoked && !self->pv->pubkey->invalid) {
@@ -380,24 +380,6 @@ seahorse_gpgme_key_refresh (SeahorseObject *obj)
 	SEAHORSE_OBJECT_CLASS (seahorse_gpgme_key_parent_class)->refresh (obj);
 }
 
-static SeahorseOperation*
-seahorse_gpgme_key_delete (SeahorseObject *obj)
-{
-	SeahorseGpgmeKey *self = SEAHORSE_GPGME_KEY (obj);
-	gpgme_error_t gerr;
-	GError *err = NULL;
-	
-	if (self->pv->seckey)
-		gerr = seahorse_gpgme_key_op_delete_pair (self);
-	else
-		gerr = seahorse_gpgme_key_op_delete (self);
-	
-	if (!GPG_IS_OK (gerr))
-		seahorse_gpgme_to_error (gerr, &err);
-	
-	return seahorse_operation_new_complete (err);
-}
-
 static GList*
 seahorse_gpgme_key_get_uids (SeahorsePgpKey *base)
 {
@@ -550,8 +532,7 @@ seahorse_gpgme_key_class_init (SeahorseGpgmeKeyClass *klass)
 	
 	seahorse_class->refresh = seahorse_gpgme_key_refresh;
 	seahorse_class->realize = seahorse_gpgme_key_realize;
-	seahorse_class->delete = seahorse_gpgme_key_delete;
-	
+
 	pgp_class->get_uids = seahorse_gpgme_key_get_uids;
 	pgp_class->set_uids = seahorse_gpgme_key_set_uids;
 	pgp_class->get_subkeys = seahorse_gpgme_key_get_subkeys;
