@@ -4,6 +4,7 @@
  * Copyright (C) 2003 Jacob Perkins
  * Copyright (C) 2005 Jim Pharis
  * Copyright (C) 2005-2006 Stefan Walter
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +30,7 @@
   
 #include <glib/gi18n.h>
 
-#include "seahorse-gconf.h"
+#include "seahorse-bind.h"
 #include "seahorse-gtkstock.h"
 #include "seahorse-object.h"
 #include "seahorse-object-model.h"
@@ -46,7 +47,8 @@
 #include "seahorse-pgp-signature.h"
 #include "seahorse-pgp-subkey.h"
 
-#include "common/seahorse-bind.h"
+#define DEBUG_FLAG SEAHORSE_DEBUG_KEYS
+#include "seahorse-debug.h"
 
 #include <time.h>
 
@@ -161,16 +163,16 @@ on_pgp_signature_row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTre
 	}
 }
 
-static GSList*
-unique_slist_strings (GSList *keyids)
+static GList*
+unique_slist_strings (GList *keyids)
 {
-    GSList *l;
+    GList *l;
     
-    keyids = g_slist_sort (keyids, (GCompareFunc)g_ascii_strcasecmp);
-    for (l = keyids; l; l = g_slist_next (l)) {
+    keyids = g_list_sort (keyids, (GCompareFunc)g_ascii_strcasecmp);
+    for (l = keyids; l; l = g_list_next (l)) {
         while (l->next && l->data && l->next->data && 
                g_ascii_strcasecmp (l->data, l->next->data) == 0)
-            keyids = g_slist_delete_link (keyids, l->next);
+            keyids = g_list_delete_link (keyids, l->next);
     }    
     
     return keyids;
@@ -279,7 +281,7 @@ on_pgp_names_revoke_clicked (GtkWidget *widget, SeahorseWidget *swidget)
         seahorse_revoke_show (SEAHORSE_PGP_KEY (skey), index - 1);
 
 #ifdef WITH_KEYSERVER
-        if (seahorse_gconf_get_boolean(AUTOSYNC_KEY) == TRUE) {
+        if (g_settings_get_boolean(AUTOSYNC_KEY) == TRUE) {
             keys = g_list_append (keys, skey);
             seahorse_keyserver_sync (keys);
             g_list_free(keys);
@@ -339,11 +341,12 @@ names_populate (SeahorseWidget *swidget, GtkTreeStore *store, SeahorsePgpKey *pk
 {
 	SeahorseObject *object;
 	GtkTreeIter uiditer, sigiter;
-	GSList *rawids = NULL;
+	GList *rawids = NULL;
 	SeahorsePgpUid *uid;
 	GList *keys, *l;
 	GList *uids, *u;
 	GList *sigs, *s;
+	GCancellable *cancellable;
 
 	/* Insert all the fun-ness */
 	uids = seahorse_pgp_key_get_uids (pkey);
@@ -366,12 +369,18 @@ names_populate (SeahorseWidget *swidget, GtkTreeStore *store, SeahorsePgpKey *pk
 			/* Never show self signatures, they're implied */
 			if (seahorse_pgp_key_has_keyid (pkey, seahorse_pgp_signature_get_keyid (s->data)))
 				continue;
-			rawids = g_slist_prepend (rawids, (gpointer)seahorse_pgp_signature_get_keyid (s->data));
+			rawids = g_list_prepend (rawids, (gpointer)seahorse_pgp_signature_get_keyid (s->data));
 		}
-        
-		/* Pass it to 'DiscoverKeys' for resolution/download */
-		keys = seahorse_context_discover_objects (SCTX_APP (), SEAHORSE_PGP, rawids);
-		g_slist_free (rawids);
+
+		/*
+		 * Pass it to 'DiscoverKeys' for resolution/download, cancellable
+		 * ties search scope together
+		 */
+		cancellable = g_cancellable_new ();
+		keys = seahorse_context_discover_objects (seahorse_context_instance (),
+		                                          SEAHORSE_PGP, rawids, cancellable);
+		g_object_unref (cancellable);
+		g_list_free (rawids);
 		rawids = NULL;
         
 		/* Add the keys to the store */
@@ -490,7 +499,7 @@ on_pgp_owner_photo_drag_received (GtkWidget *widget, GdkDragContext *context, gi
      * and there has to be a better way to get rid of the trailing \r\n appended
      * to the end of the path after the call to g_filename_from_uri
      */
-    if((sel_data != NULL) && (sel_data->length >= 0)) {
+    if((sel_data != NULL) && (gtk_selection_data_get_length (sel_data) >= 0)) {
         g_return_if_fail (target_type == TARGET_URI);
         
         uri_list = gtk_selection_data_get_uris (sel_data);
@@ -1003,9 +1012,8 @@ export_complete (GFile *file, GAsyncResult *result, gchar *contents)
 	if (!g_file_replace_contents_finish (file, result, NULL, &err)) {
 		uri = g_file_get_uri (file);
 		unesc_uri = g_uri_unescape_string (seahorse_util_uri_get_last (uri), NULL);
-        seahorse_util_handle_error (err, _("Couldn't export key to \"%s\""),
-                                    unesc_uri);
-        g_clear_error (&err);
+		seahorse_util_handle_error (&err, NULL, _("Couldn't export key to \"%s\""),
+		                            unesc_uri);
         g_free (uri);
         g_free (unesc_uri);
 	}
@@ -1056,9 +1064,8 @@ on_pgp_details_export_button (GtkWidget *widget, SeahorseWidget *swidget)
 		                               G_FILE_CREATE_PRIVATE, NULL, 
 		                               (GAsyncReadyCallback)export_complete, results);
 	} else {
-		seahorse_gpgme_to_error (gerr, &err);
-		seahorse_util_handle_error (err, _("Couldn't export key."));
-		g_clear_error (&err);
+		seahorse_gpgme_propagate_error (gerr, &err);
+		seahorse_util_handle_error (&err, NULL, _("Couldn't export key."));
 	}
 	
 	g_free (uri);
@@ -1110,8 +1117,8 @@ setup_details_trust (SeahorseWidget *swidget)
     GtkTreeIter iter;
     GtkCellRenderer *text_cell = gtk_cell_renderer_text_new ();
 
-    DBG_PRINT(("KeyProperties: Setting up Trust Combo Box Store\n"));
-    
+    seahorse_debug ("KeyProperties: Setting up Trust Combo Box Store");
+
     object = SEAHORSE_OBJECT_WIDGET (swidget)->object;
     etype = seahorse_object_get_usage (object);
     
@@ -1162,7 +1169,7 @@ setup_details_trust (SeahorseWidget *swidget)
     
     gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (model));                                        
 
-    DBG_PRINT(("KeyProperties: Finished Setting up Trust Combo Box Store\n"));
+    seahorse_debug ("KeyProperties: Finished Setting up Trust Combo Box Store");
 }
 
 static void
@@ -1475,9 +1482,10 @@ signatures_populate_model (SeahorseWidget *swidget, SeahorseObjectModel *skmodel
 	GtkTreeIter iter;
 	GtkWidget *widget;
 	gboolean have_sigs = FALSE;
-	GSList *rawids = NULL;
+	GList *rawids = NULL;
 	GList *keys, *l, *uids;
 	GList *sigs, *s;
+	GCancellable *cancellable;
 
 	pkey = SEAHORSE_PGP_KEY (SEAHORSE_OBJECT_WIDGET (swidget)->object);
 	widget = GTK_WIDGET (seahorse_widget_get_widget (swidget, "signatures-tree"));
@@ -1494,7 +1502,7 @@ signatures_populate_model (SeahorseWidget *swidget, SeahorseObjectModel *skmodel
 			if (seahorse_pgp_key_has_keyid (pkey, seahorse_pgp_signature_get_keyid (s->data)))
 				continue;
 			have_sigs = TRUE;
-			rawids = g_slist_prepend (rawids, (gpointer)seahorse_pgp_signature_get_keyid (s->data));
+			rawids = g_list_prepend (rawids, (gpointer)seahorse_pgp_signature_get_keyid (s->data));
 		}
 	}
     
@@ -1505,10 +1513,16 @@ signatures_populate_model (SeahorseWidget *swidget, SeahorseObjectModel *skmodel
     
 		/* String out duplicates */
 		rawids = unique_slist_strings (rawids);
-        
-		/* Pass it to 'DiscoverKeys' for resolution/download */
-		keys = seahorse_context_discover_objects (SCTX_APP (), SEAHORSE_PGP, rawids);
-		g_slist_free (rawids);
+
+		/*
+		 * Pass it to 'DiscoverKeys' for resolution/download. cancellable ties
+		 * search scope together
+		 */
+		cancellable = g_cancellable_new ();
+		keys = seahorse_context_discover_objects (seahorse_context_instance (), SEAHORSE_PGP,
+		                                          rawids, cancellable);
+		g_object_unref (cancellable);
+		g_list_free (rawids);
 		rawids = NULL;
         
 		/* Add the keys to the store */
@@ -1635,12 +1649,11 @@ do_trust (SeahorseWidget *swidget)
     /* Local keys */
     } else {
         guint trust;
-        gboolean trusted, managed;
+        gboolean managed;
         const gchar *icon = NULL;
         
         trust = seahorse_pgp_key_get_trust (pkey);
-    
-        trusted = FALSE;
+
         managed = FALSE;
         
         switch (trust) {
@@ -1652,14 +1665,12 @@ do_trust (SeahorseWidget *swidget)
         
         /* Trust is specified manually */
         case SEAHORSE_VALIDITY_ULTIMATE:
-            trusted = TRUE;
             managed = FALSE;
             icon = SEAHORSE_STOCK_SIGN_OK;
             break;
         
         /* Trust is specified manually */
         case SEAHORSE_VALIDITY_NEVER:
-            trusted = FALSE;
             managed = FALSE;
             icon = SEAHORSE_STOCK_SIGN_BAD;
             break;
@@ -1667,14 +1678,12 @@ do_trust (SeahorseWidget *swidget)
         /* We manage the trust through this page */
         case SEAHORSE_VALIDITY_FULL:
         case SEAHORSE_VALIDITY_MARGINAL:
-            trusted = TRUE;
             managed = TRUE;
             icon = SEAHORSE_STOCK_SIGN_OK;
             break;
         
         /* We manage the trust through this page */
         case SEAHORSE_VALIDITY_UNKNOWN:
-            trusted = FALSE;
             managed = TRUE;
             icon = SEAHORSE_STOCK_SIGN;
             break;
