@@ -24,12 +24,11 @@
 
 #include <glib/gi18n.h>
 
-#include "seahorse-source.h"
 #include "seahorse-gtkstock.h"
+#include "seahorse-object-list.h"
+#include "seahorse-source.h"
 #include "seahorse-util.h"
 
-#include "common/seahorse-object-list.h"
-
 #include "pgp/seahorse-pgp.h"
 #include "pgp/seahorse-pgp-key.h"
 #include "pgp/seahorse-pgp-uid.h"
