@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2003 Jacob Perkins
  * Copyright (C) 2004, 2005, 2006 Stefan Walter
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,20 +29,23 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include "seahorse-gconf.h"
 #include "seahorse-key-manager-store.h"
 #include "seahorse-preferences.h"
 #include "seahorse-util.h"
 #include "seahorse-validity.h"
 
+#define DEBUG_FLAG SEAHORSE_DEBUG_DRAG
+#include "seahorse-debug.h"
+
 #include "eggtreemultidnd.h"
 
 #define KEY_MANAGER_SORT_KEY "/apps/seahorse/listing/sort_by"
 
 enum {
-    PROP_0,
-    PROP_MODE,
-    PROP_FILTER
+	PROP_0,
+	PROP_MODE,
+	PROP_FILTER,
+	PROP_SETTINGS
 };
 
 enum {
@@ -88,10 +92,11 @@ static GtkTargetEntry store_targets[] = {
 	{ "XdndDirectSave0", 0, DRAG_INFO_XDS }
 };
 
-struct _SeahorseKeyManagerStorePriv {    
-    GtkTreeModelFilter      *filter;
-    GtkTreeModelSort        *sort;
-    
+struct _SeahorseKeyManagerStorePriv {
+	GtkTreeModelFilter *filter;
+	GtkTreeModelSort *sort;
+	GSettings *settings;
+
     SeahorseKeyManagerStoreMode    filter_mode;
     gchar*                  filter_text;
     guint                   filter_stag;
@@ -314,69 +319,64 @@ set_sort_to (SeahorseKeyManagerStore *skstore, const gchar *name)
 static void
 sort_changed (GtkTreeSortable *sort, gpointer user_data)
 {
-    gint id;
-    GtkSortType ord;
-    SeahorseKeyManagerStore *skstore;
-    const gchar* t;
-    gchar* x;
-    
-    skstore = SEAHORSE_KEY_MANAGER_STORE (user_data);
-    g_return_if_fail (skstore->priv->sort);
-    
-    /* We have a sort so save it */
-    if (gtk_tree_sortable_get_sort_column_id (sort, &id, &ord)) {
-        if (id >= 0 && id < N_COLS) {
-            t = column_info[id].data;
-            if (t != NULL) {
-                x = g_strconcat (ord == GTK_SORT_DESCENDING ? "-" : "", t, NULL);
-                seahorse_gconf_set_string (KEY_MANAGER_SORT_KEY, x);
-                g_free (x);
-            }
-        }
-    }
-    
-    /* No sort so save blank */
-    else {
-        seahorse_gconf_set_string (KEY_MANAGER_SORT_KEY, "");
-    }
+	SeahorseKeyManagerStore *self = SEAHORSE_KEY_MANAGER_STORE (user_data);
+	GtkSortType ord;
+	gchar* value;
+	gint column_id;
+
+	g_return_if_fail (self->priv->sort);
+	if (!self->priv->settings)
+		return;
+
+	/* We have a sort so save it */
+	if (gtk_tree_sortable_get_sort_column_id (sort, &column_id, &ord)) {
+		if (column_id >= 0 && column_id < N_COLS) {
+			if (column_info[column_id].data != NULL) {
+				value = g_strconcat (ord == GTK_SORT_DESCENDING ? "-" : "",
+				                     column_info[column_id].data, NULL);
+				g_settings_set_string (self->priv->settings, "sort-by", value);
+				g_free (value);
+			}
+		}
+
+	/* No sort so save blank */
+	} else if (self->priv->settings) {
+		g_settings_set_string (self->priv->settings, "sort-by", "");
+	}
 }
 
 static void
-gconf_notification (GConfClient *gclient, guint id, GConfEntry *entry, GtkTreeView *view)
+on_manager_settings_changed (GSettings *settings, const gchar *key, gpointer user_data)
 {
-    SeahorseKeyManagerStore *skstore;
-    GtkTreeViewColumn *col = NULL;
-    GList *columns, *l;
-    GConfValue *value;
-    const gchar *key;
-    const gchar *t;
+	SeahorseKeyManagerStore *self;
+	GtkTreeViewColumn *col = NULL;
+	GtkTreeView *view = GTK_TREE_VIEW (user_data);
+	const gchar *col_key = NULL;
+	GList *columns, *l;
+	gchar *sort_by;
 
-    key = gconf_entry_get_key (entry);
+	if (g_str_equal (key, "sort-by")) {
+		self = key_store_from_model (gtk_tree_view_get_model (view));
+		g_return_if_fail (SEAHORSE_IS_KEY_MANAGER_STORE (self));
+		sort_by = g_settings_get_string (settings, key);
+		set_sort_to (self, sort_by);
+		g_free (sort_by);
+		return;
+	}
 
-    g_assert (key != NULL);
-    g_assert (GTK_IS_TREE_VIEW (view));
-    
-    if (g_str_equal (key, KEY_MANAGER_SORT_KEY)) {
-        skstore = key_store_from_model (gtk_tree_view_get_model (view));
-        g_return_if_fail (SEAHORSE_IS_KEY_MANAGER_STORE (skstore));
-        set_sort_to (skstore, gconf_value_get_string (gconf_entry_get_value (entry)));
-    }
-    
-    columns = gtk_tree_view_get_columns (view);
-    for (l = columns; l; l = g_list_next (l)) {
-        t = (const gchar*)g_object_get_data (G_OBJECT (l->data), "gconf-key");
-        if (t && g_str_equal (t, key)) {
-            col = GTK_TREE_VIEW_COLUMN (l->data);
-            break;
-        }
-    }
-    
-    if (col != NULL) {
-        value = gconf_entry_get_value (entry);
-        gtk_tree_view_column_set_visible (col, gconf_value_get_bool (value));
-    }
-    
-    g_list_free (columns);
+	columns = gtk_tree_view_get_columns (view);
+	for (l = columns; l; l = g_list_next (l)) {
+		col_key = g_object_get_data (G_OBJECT (l->data), "settings-key");
+		if (col_key && g_str_equal (col_key, key)) {
+			col = GTK_TREE_VIEW_COLUMN (l->data);
+			break;
+		}
+	}
+
+	if (col != NULL)
+		gtk_tree_view_column_set_visible (col, g_settings_get_boolean (settings, key));
+
+	g_list_free (columns);
 }
 
 static GtkTreeViewColumn*
@@ -471,9 +471,9 @@ drag_begin (GtkWidget *widget, GdkDragContext *context, SeahorseKeyManagerStore
 	
 	#if GTK_CHECK_VERSION (2,22,0)
         GdkWindow *source_window;
-    #endif
-    
-	DBG_PRINT (("drag_begin -->\n"));
+#endif
+
+	seahorse_debug ("drag_begin -->");
 
 	g_free (skstore->priv->drag_destination);
 	skstore->priv->drag_destination = NULL;
@@ -497,87 +497,39 @@ drag_begin (GtkWidget *widget, GdkDragContext *context, SeahorseKeyManagerStore
         }
     #endif
 	
-	DBG_PRINT (("drag_begin <--\n"));
+	seahorse_debug ("drag_begin <--");
 	return skstore->priv->drag_objects ? TRUE : FALSE;
 }
 
-static gboolean
-export_keys_to_output (GList *objects, GOutputStream *output, GError **error)
-{
-	SeahorseMultiOperation *mop = NULL;
-	SeahorseOperation *op;
-	SeahorseSource *sksrc;
-	SeahorseObject *sobj;
-	GList *next;
-	gboolean ret;
-	
-	objects = seahorse_util_objects_sort (objects);
-	DBG_PRINT (("exporting %d objects\n", g_list_length (objects)));
-	
-	while (objects) {
-		
-		/* Break off one set (same keysource) */
-		next = seahorse_util_objects_splice (objects);
+typedef struct {
+	SeahorseKeyManagerStore *skstore;
 
-		g_assert (SEAHORSE_IS_OBJECT (objects->data));
-		sobj = SEAHORSE_OBJECT (objects->data);
-
-		/* Export from this key source */        
-		sksrc = seahorse_object_get_source (sobj);
-		g_return_val_if_fail (sksrc != NULL, FALSE);
-
-		if (!mop) 
-			mop = seahorse_multi_operation_new ();
-
-		/* We pass our own data object, to which data is appended */
-		op = seahorse_source_export (sksrc, objects, output);
-		g_return_val_if_fail (op != NULL, FALSE);
-
-		seahorse_multi_operation_take (mop, op);
-		
-		g_list_free (objects);
-		objects = next;		
-	}
-	
-	/* Make sure it's complete before we can return data */
-	op = SEAHORSE_OPERATION (mop);
-	seahorse_operation_wait (op);
-
-	ret = TRUE;
-	if (!seahorse_operation_is_successful (op)) { 
-		seahorse_operation_copy_error (op, error);
-		ret = FALSE;
-	}
-	
-	g_object_unref (mop);
-	return ret;
-}
+	gint exports;
+	gboolean failures;
+} export_keys_to_output_closure;
 
 static gboolean
-export_to_text (SeahorseKeyManagerStore *skstore, GtkSelectionData *selection_data)
+export_to_text (SeahorseKeyManagerStore *skstore,
+                GtkSelectionData *selection_data)
 {
 	GOutputStream *output;
-	gboolean ret;
-	GList *keys;
-	
-	ret = FALSE;
-	
-	g_return_val_if_fail (skstore->priv->drag_objects, FALSE);
-	keys = g_list_copy (skstore->priv->drag_objects);
+	gboolean ret = FALSE;
 
-	DBG_PRINT (("exporting to text\n"));
+	g_return_val_if_fail (skstore->priv->drag_objects, FALSE);
+	seahorse_debug ("exporting to text");
 
 	output = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
 	g_return_val_if_fail (output, FALSE);
 
-	/* This modifies and frees keys */
-	ret = export_keys_to_output (keys, output, &skstore->priv->drag_error) &&
-	       g_output_stream_close (output, NULL, &skstore->priv->drag_error);
+	ret = seahorse_source_export_auto_wait (skstore->priv->drag_objects, output,
+	                                        &skstore->priv->drag_error) &&
+	      g_output_stream_close (output, NULL, &skstore->priv->drag_error);
+
 	if (ret) {
-		DBG_PRINT (("setting selection text\n"));
+		seahorse_debug ("setting selection text");
 		gtk_selection_data_set_text (selection_data, 
 		                             g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output)), 
-		                             seahorse_util_memory_output_length (G_MEMORY_OUTPUT_STREAM (output)));
+		                             g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (output)));
 	} else {
 		g_message ("error occurred on export: %s", 
 		           skstore->priv->drag_error && skstore->priv->drag_error->message ? 
@@ -596,8 +548,8 @@ export_to_filename (SeahorseKeyManagerStore *skstore, const gchar *filename)
 	gchar *uri;
 	GFile *file;
 	GList *keys;
-	
-	DBG_PRINT (("exporting to %s\n", filename));
+
+	seahorse_debug ("exporting to %s", filename);
 
 	ret = FALSE;
 	g_return_val_if_fail (skstore->priv->drag_objects, FALSE);
@@ -614,9 +566,9 @@ export_to_filename (SeahorseKeyManagerStore *skstore, const gchar *filename)
 	
 	if (output) {
 		/* This modifies and frees keys */
-		ret = export_keys_to_output (keys, output, &skstore->priv->drag_error) &&
+		ret = seahorse_source_export_auto_wait (keys, output, &skstore->priv->drag_error) &&
 		      g_output_stream_close (output, NULL, &skstore->priv->drag_error);
-		
+
 		g_object_unref (output);
 	}
 	
@@ -630,13 +582,13 @@ drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *sel
 	gchar *destination;
 	gboolean ret;
 
-	DBG_PRINT (("drag_data_get %d -->\n", info)); 
+	seahorse_debug ("drag_data_get %d -->", info);
 	
 	g_return_val_if_fail (skstore->priv->drag_objects, FALSE);
 
 	/* The caller wants plain text */
 	if (info == DRAG_INFO_TEXT) {
-		DBG_PRINT (("returning object text\n"));
+		seahorse_debug ("returning object text");
 		export_to_text (skstore, selection_data);
 		
 	/* The caller wants XDS */
@@ -654,10 +606,10 @@ drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *sel
 		
 	/* Unrecognized format */
 	} else {
-		DBG_PRINT (("Unrecognized format: %d\n", info));
+		seahorse_debug ("Unrecognized format: %d", info);
 	}
 	
-	DBG_PRINT(("drag_data_get <--\n"));
+	seahorse_debug ("drag_data_get <--");
 	return ret;
 }
 
@@ -665,9 +617,9 @@ static void
 drag_end (GtkWidget *widget, GdkDragContext *context, SeahorseKeyManagerStore *skstore)
 {
 	gchar *filename, *name;
-	
-	DBG_PRINT (("drag_end -->\n"));
-	
+
+	seahorse_debug ("drag_end -->");
+
 	if (skstore->priv->drag_destination && !skstore->priv->drag_error) {
 		g_return_if_fail (skstore->priv->drag_objects);
 	
@@ -691,8 +643,8 @@ drag_end (GtkWidget *widget, GdkDragContext *context, SeahorseKeyManagerStore *s
 	skstore->priv->drag_objects = NULL;
 	g_free (skstore->priv->drag_destination);
 	skstore->priv->drag_destination = NULL;
-	
-	DBG_PRINT (("drag_end <--\n"));
+
+	seahorse_debug ("drag_end <--");
 }
 
 static gint 
@@ -726,7 +678,11 @@ seahorse_key_manager_store_get_property (GObject *gobject, guint prop_id,
     case PROP_MODE:
         g_value_set_uint (value, skstore->priv->filter_mode);
         break;
-        
+
+    case PROP_SETTINGS:
+        g_value_set_object (value, skstore->priv->settings);
+        break;
+
     /* The filter text. Note that we act as if we don't have any 
      * filter text when not in filtering mode */
     case PROP_FILTER:
@@ -755,7 +711,11 @@ seahorse_key_manager_store_set_property (GObject *gobject, guint prop_id,
             refilter_later (skstore);
         }
         break;
-        
+
+    case PROP_SETTINGS:
+        skstore->priv->settings = g_value_dup_object (value);
+        break;
+
     /* The filter text */
     case PROP_FILTER:
         t = g_value_get_string (value);
@@ -823,6 +783,10 @@ seahorse_key_manager_store_class_init (SeahorseKeyManagerStoreClass *klass)
     g_object_class_install_property (gobject_class, PROP_FILTER,
         g_param_spec_string ("filter", "Key Store Filter", "Key store filter for when in filtered mode",
                              "", G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class, PROP_SETTINGS,
+        g_param_spec_object ("settings", "Settings", "Manager Settings",
+                             G_TYPE_SETTINGS, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 /* -----------------------------------------------------------------------------
@@ -830,16 +794,16 @@ seahorse_key_manager_store_class_init (SeahorseKeyManagerStoreClass *klass)
  */
 
 SeahorseKeyManagerStore*
-seahorse_key_manager_store_new (SeahorseSet *skset, GtkTreeView *view)
+seahorse_key_manager_store_new (SeahorseSet *skset, GtkTreeView *view, GSettings *settings)
 {
     SeahorseKeyManagerStore *skstore;
     GtkTreeViewColumn *col;
     SeahorseObjectPredicate *pred;
     GtkCellRenderer *renderer;
-    gchar *sort;
+    gchar *sort_by;
     gint last;
-    
-    skstore = g_object_new (SEAHORSE_TYPE_KEY_MANAGER_STORE, "set", skset, NULL);
+
+    skstore = g_object_new (SEAHORSE_TYPE_KEY_MANAGER_STORE, "set", skset, "settings", settings, NULL);
     last = seahorse_set_model_set_columns (SEAHORSE_SET_MODEL (skstore), column_info, N_COLS);
     g_return_val_if_fail (last == N_COLS - 1, NULL);
 
@@ -881,42 +845,41 @@ seahorse_key_manager_store_new (SeahorseSet *skset, GtkTreeView *view)
     /* Public keys show validity */
     if (pred->usage == SEAHORSE_USAGE_PUBLIC_KEY) {
         col = append_text_column (skstore, view, _("Validity"), COL_VALIDITY_STR);
-        g_object_set_data (G_OBJECT (col), "gconf-key", SHOW_VALIDITY_KEY);
-        gtk_tree_view_column_set_visible (col, seahorse_gconf_get_boolean (SHOW_VALIDITY_KEY));
+        g_object_set_data (G_OBJECT (col), "settings-key", "show-validity");
+        gtk_tree_view_column_set_visible (col, g_settings_get_boolean (settings, "show-validity"));
         gtk_tree_view_column_set_sort_column_id (col, COL_VALIDITY);
     }
 
     /* Trust column */
     col = append_text_column (skstore, view, _("Trust"), COL_TRUST_STR);
-    g_object_set_data (G_OBJECT (col), "gconf-key", SHOW_TRUST_KEY);
-    gtk_tree_view_column_set_visible (col, seahorse_gconf_get_boolean (SHOW_TRUST_KEY));
+    g_object_set_data (G_OBJECT (col), "settings-key", "show-trust");
+    gtk_tree_view_column_set_visible (col, g_settings_get_boolean (settings, "show-trust"));
     gtk_tree_view_column_set_sort_column_id (col, COL_TRUST);
-    
+
     /* The key type column */
     col = append_text_column (skstore, view, _("Type"), COL_TYPE);
-    g_object_set_data (G_OBJECT (col), "gconf-key", SHOW_TYPE_KEY);
-    gtk_tree_view_column_set_visible (col, seahorse_gconf_get_boolean (SHOW_TYPE_KEY));
+    g_object_set_data (G_OBJECT (col), "settings-key", "show-type");
+    gtk_tree_view_column_set_visible (col, g_settings_get_boolean (settings, "show-type"));
     gtk_tree_view_column_set_sort_column_id (col, COL_TYPE);
 
     /* Expiry date column */
     col = append_text_column (skstore, view, _("Expiration Date"), COL_EXPIRES_STR);
-    g_object_set_data (G_OBJECT (col), "gconf-key", SHOW_EXPIRES_KEY);
-    gtk_tree_view_column_set_visible (col, seahorse_gconf_get_boolean (SHOW_EXPIRES_KEY));
+    g_object_set_data (G_OBJECT (col), "settings-key", "show-expiry");
+    gtk_tree_view_column_set_visible (col, g_settings_get_boolean (settings, "show-expiry"));
     gtk_tree_view_column_set_sort_column_id (col, COL_EXPIRES);
 
     /* Also watch for sort-changed on the store */
     g_signal_connect (skstore->priv->sort, "sort-column-changed", G_CALLBACK (sort_changed), skstore);
 
     /* Update sort order in case the sorted column was added */
-    if ((sort = seahorse_gconf_get_string (KEY_MANAGER_SORT_KEY)) != NULL) {
-        set_sort_to (skstore, sort);
-        g_free (sort);
+    if ((sort_by = g_settings_get_string (settings, "sort-by")) != NULL) {
+        set_sort_to (skstore, sort_by);
+        g_free (sort_by);
     } 
     
     gtk_tree_view_set_enable_search (view, FALSE);
-    
-    seahorse_gconf_notify_lazy (LISTING_SCHEMAS, (GConfClientNotifyFunc)gconf_notification, 
-                                view, GTK_WIDGET (view));
+
+    g_signal_connect_object (settings, "changed", G_CALLBACK (on_manager_settings_changed), view, 0);
 
     /* Tree drag */
     egg_tree_multi_drag_add_drag_support (view);    
