@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2005 Stefan Walter
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,6 @@
 #include <gtk/gtk.h>
 
 #include "seahorse-object.h"
-#include "seahorse-operation.h"
 #include "seahorse-source.h"
 #include "seahorse-validity.h"
 
@@ -73,8 +73,15 @@ const gchar*            seahorse_ssh_key_get_algo_str         (SeahorseSSHKey *s
 guint                   seahorse_ssh_key_get_strength         (SeahorseSSHKey *skey);
 
 const gchar*            seahorse_ssh_key_get_location         (SeahorseSSHKey *skey);
-                                                                   
-SeahorseOperation*      seahorse_ssh_key_op_change_passphrase (SeahorseSSHKey *skey);
+
+void             seahorse_ssh_key_op_change_passphrase_async  (SeahorseSSHKey *self,
+                                                               GCancellable *cancellable,
+                                                               GAsyncReadyCallback callback,
+                                                               gpointer user_data);
+
+gboolean         seahorse_ssh_key_op_change_passphrase_finish (SeahorseSSHKey *self,
+                                                               GAsyncResult *result,
+                                                               GError **error);
 
 SeahorseValidity        seahorse_ssh_key_get_trust            (SeahorseSSHKey *self);
 
